@@ -26,12 +26,12 @@ class MockScenario:
 
     def var(self, name, **kwargs):
         df = self.data["aeei"]
-        # add extra commodity to be removed
+        # Add extra commodity to be removed
         extra_commod = df[df.sector == "i_therm"].copy()
-        extra_commod["sector"] = self.data["config"]["ignore_sectors"][0]
-        # add extra region to be removed
+        extra_commod["sector"] = "bar"
+        # Add extra region to be removed
         extra_region = df[df.node == "R11_AFR"].copy()
-        extra_region["node"] = self.data["config"]["ignore_nodes"][0]
+        extra_region["node"] = "foo"
         df = pd.concat([df, extra_commod, extra_region])
 
         if name == "DEMAND":
@@ -73,6 +73,24 @@ def test_calc_valid_data_dict(westeros_solved):
     c.read_data()
 
 
+# Test for selecting desirable years specified in config from the Excel input
+def test_calc_valid_years(westeros_solved):
+    s = westeros_solved
+    data = pd.read_excel(W_DATA_PATH, sheet_name=None, engine="openpyxl")
+    # Adding an arbitrary year
+    arbitrary_yr = 2021
+    gdp_extra_yr = data["gdp_calibrate"].iloc[0, :].copy()
+    gdp_extra_yr["year"] = arbitrary_yr
+    data["gdp_calibrate"] = data["gdp_calibrate"].append(gdp_extra_yr)
+    # Check the arbitrary year is not in config
+    assert arbitrary_yr not in data["config"]["year"]
+    # But it is in gdp_calibrate
+    assert arbitrary_yr in set(data["gdp_calibrate"]["year"])
+    # And macro does calibration without error
+    c = macro.Calculate(s, data)
+    c.read_data()
+
+
 def test_calc_no_solution(westeros_not_solved):
     s = westeros_not_solved
     pytest.raises(RuntimeError, macro.Calculate, s, W_DATA_PATH)
@@ -81,11 +99,24 @@ def test_calc_no_solution(westeros_not_solved):
 def test_config(westeros_solved):
     s = westeros_solved
     c = macro.Calculate(s, W_DATA_PATH)
-    c.nodes = set(list(c.nodes) + ["foo"])
-    c.sectors = set(list(c.sectors) + ["bar"])
+    assert "config" in c.data
+    assert "sector" in c.data["config"]
+
+    # Removing a column from config and testing
+    data = c.data.copy()
+    data["config"] = c.data["config"][["node", "sector"]]
+    try:
+        macro.Calculate(s, data)
+    except KeyError as error:
+        assert 'Missing config data for "level"' in str(error)
+
+    # Removing config completely and testing
+    data.pop("config")
+    try:
+        macro.Calculate(s, data)
+    except KeyError as error:
+        assert "Missing config in input data" in str(error)
 
-    assert c.nodes == set(["Westeros", "foo"])
-    assert c.sectors == set(["light", "bar"])
     c.read_data()
     assert c.nodes == set(["Westeros"])
     assert c.sectors == set(["light"])
@@ -133,12 +164,9 @@ def test_calc_data_missing_datapoint(westeros_solved):
         (
             "_total_cost",
             "allclose",
-            1e-3
-            * np.array(
-                [9.17583, 11.653576414880422, 12.446804289049216, 15.369457033651215]
-            ),
+            1e-3 * np.array([6.18242, 8.6601720, 13.4040172, 14.9067117]),
         ),
-        ("_price", "allclose", [211, 511.02829331, 162.03953933, 161.0026274]),
+        ("_price", "allclose", [211, 511.0282933, 162.0395393, 161.0026274]),
         ("_demand", "allclose", [27, 55, 82, 104]),
         ("_bconst", "allclose", [9.68838201e-08]),
         ("_aconst", "allclose", [26.027323]),
@@ -154,6 +182,37 @@ def test_calc(westeros_solved, method, test, expected):
     assertion(function().values, expected)
 
 
+# Testing how macro handles zero values in PRICE_COMMODITY
+def test_calc_price_zero(westeros_solved):
+    s = westeros_solved
+    clone = s.clone(scenario="low_demand", keep_solution=False)
+    clone.check_out()
+    # Lowering demand in the first year
+    clone.add_par("demand", ["Westeros", "light", "useful", 700, "year"], 10, "GWa")
+    # Making investment and var cost zero for delivering light
+    # TODO: these units are based on testing.make_westeros: needs improvement
+    clone.add_par("inv_cost", ["Westeros", "bulb", 700], 0, "USD/GWa")
+    for y in [690, 700]:
+        clone.add_par(
+            "var_cost", ["Westeros", "grid", y, 700, "standard", "year"], 0, "USD/GWa"
+        )
+
+    clone.commit("demand reduced and zero cost for bulb")
+    clone.solve()
+    price = clone.var("PRICE_COMMODITY")
+    # Assert if there is no zero price (to make sure MACRO receives 0 price)
+    assert np.isclose(0, price["lvl"]).any()
+    c = macro.Calculate(clone, W_DATA_PATH)
+    c.read_data()
+    try:
+        c._price()
+    except RuntimeError as err:
+        # To make sure the right error message is raised in macro.py
+        assert "0-price found in MESSAGE variable PRICE_" in str(err)
+    else:
+        raise Exception("No error in macro.read_data() for zero price(s)")
+
+
 def test_init(message_test_mp):
     scen = Scenario(message_test_mp, **SCENARIO["dantzig"])
 
@@ -213,7 +272,7 @@ def test_calibrate_roundtrip(westeros_solved):
     aeei = with_macro.par("aeei")["value"].values
     npt.assert_allclose(
         aeei,
-        1e-3 * np.array([20, -7.5142879, 43.6256281, 21.1434631]),
+        1e-3 * np.array([20, -7.5142661, 43.6256127, 21.14346]),
     )
     grow = with_macro.par("grow")["value"].values
     npt.assert_allclose(
@@ -222,9 +281,9 @@ def test_calibrate_roundtrip(westeros_solved):
         * np.array(
             [
                 26.5836313,
-                69.1417640,
-                79.1435807,
-                24.5225556,
+                69.1417537,
+                79.1435885,
+                24.5225555,
             ]
         ),
     )
